\name{Ollerenshaw}
\alias{Ollerenshaw}
\docType{data}
\title{A most perfect square due to Ollerenshaw}
\description{
A 12-by-12 most perfect square due to Ollerenshaw
}
\usage{data(Ollerenshaw)}
\source{
  ``Most perfect pandiagonal magic squares'', K. Ollerenshaw and D. Bree,
  1998, Institute of Mathematics and its applications}
 
}
\examples{
data(Ollerenshaw)
is.mostperfect(Ollerenshaw)
}
\keyword{datasets}

\eof
\name{allsubhypercubes}
\alias{allsubhypercubes}
\title{Subhypercubes of magic hypercubes}
\description{
Extracts all subhypercubes from an n-dimensional hypercube.
}
\usage{
allsubhypercubes(a)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{The magic hypercube whose subhypercubes are computed}
}
\value{
  Returns a list, each element of which is a subhypercube.  Note that
  major diagonals are also returned (as n-by-1 arrays).

  The names of the list are the extracted subhypercubes.  Consider
  \code{a <- magichypercube.4n(1,d=4)} (so n=4) and if \code{jj <-
  allsubhypercubes(a)}, consider \code{jj[9]}.  The name of
  \code{jj[9]} is \code{"n-i+1,i,i,"}; its value is a square matrix.  The
  columns of \code{jj[9]} may be recovered by \code{a[n-i+1,i,i,]}
  with \eqn{i=1\ldots n}{i=1...n} (NB: that is, \code{rbind(a[n-1+1,1,1,],
  a[n-2+1,2,2,], a[n-3+1,3,3,], a[n-4+1,4,4,])} where \code{n=4}).

  The list does not inlude the whole array.
}
\author{Robin K. S. Hankin}
\note{This function is a dog's dinner.  It's complicated, convoluted,
  and needs an absurd use of the \code{eval(parse(text=...))}
  construction.  Basically it sucks big time.

  BUT\ldots I cannot for the life of me see a better way that gives the
  same results, without loops, on hypercubes of arbitrary dimension.

  On my 256MB Linuxbox, \code{allsubhypercubes()} cannot cope with
  \code{d} as high as 5, for \code{n=4}.  Heigh ho.
  }
\seealso{\code{\link{is.perfect}}}
\examples{
  a <- magichypercube.4n(1,d=4)
  allsubhypercubes(a)
}
\keyword{array}

\eof
\name{allsums}
\alias{allsums}
\title{Row, column, and two diagonal sums of arrays}
\description{
  Returns all rowsums, all columnsums, and all (broken) diagonal sums of
  a putative magic square.
}
\usage{
allsums(m)
}
\arguments{
  \item{m}{The square to be tested}
}
\value{
  Returns a list of four elements.
  \item{rowsums}{All n row sums}
  \item{colsums}{All n column sums}
  \item{majors}{All n broken major diagonals (northwest-southeast).
    First element is the long (unbroken) major diagonal, tested by
    \code{is.magic()}}
  \item{minors}{All n broken minor diagonals (northeast-southwest).
    First element is the long (unbroken) minor diagonal.}
}
\author{Robin K. S. Hankin}
\note{Used by \code{is.magic()} et seq}
\seealso{\code{\link{is.magic}},\
\code{\link{is.semimagic}},\
\code{\link{is.panmagic}}
  }
\examples{
allsums(magic.prime(7))
}
\keyword{array}

\eof
\name{as.standard}
\alias{as.standard}
\alias{is.standard}

\title{Standard form for magic squares}
\description{
  Transforms a magic square or magic hypercube into Frenicle's standard form
}
\usage{
as.standard(a)
is.standard(a)
}
\arguments{
  \item{a}{Magic square or hypercube (array) to be tested or transformed}
}

\details{
For a square, \code{as.standard()} transforms a magic square into
Frenicle's standard form.  The four numbers at each of the four corners
are determined.  First, the square is rotated so the smallest of the
four is at the upper left.  Then, element \code{[1,2]} is compared with
element\code{[2,1]} and, if it is larger, the transpose is taken.

Thus all eight rotated and transposed versions of a magic square have
the same standard form.

The square returned by \code{magic()} is in standard form.

For hypercubes, the algorithm is generalized.  Firstly, the hypercube is
reflected so that \code{a[1,1,...,1,1]} is the smallest of the
\eqn{2^d}{2^d} corner elements (eg \code{a[1,n,1,...,1,1]}).

Next, \code{aperm()} is called so that

\code{a[1,1,...,1,2] < a[1,1,...,2,1] < ... < a[2,1,...,1,1]}.

Note that the inequalities are strict as hypercubes are assumed to be
normal.
  
\code{is.standard()} returns \code{TRUE} if the magic square or
hypercube is in standard form.  \code{is.standard()} and
\code{as.standard()} check for neither magicness nor normality (use
\code{\link{is.magic}} and \code{\link{is.normal}} for this).
}

\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}},\ \code{\link{eq}}}
\examples{
is.standard(magic.2np1(4))
as.standard(magic.4n(3))

as.standard(magichypercube.4n(1,5))

}
\keyword{array}

\eof
\name{cube2}
\alias{cube2}
\docType{data}
\title{A pantriagonal magic cube}
\description{
A pantriagonal magic cube of order 4 originally due to Hendricks
}
\usage{data(cube2)}
\details{
  Meaning of `pantriagonal' currently unclear
}
\source{
Hendricks
}
\examples{
data(cube2)
is.magichypercube(cube2)
is.perfect(cube2)
}
\keyword{datasets}

\eof
\name{diag.off}
\alias{diag.off}
\title{Extracts broken diagonals}
\description{
  Returns broken diagonals of a magic square
}
\usage{
diag.off(a, offset = 0, nw.se = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{Square matrix}
  \item{offset}{vertical offset} 
  \item{nw.se}{Boolean variable with TRUE meaning trace diagonals along
    the northwest-southeast direction (point \code{[1,n]}
    to \code{[n,1]}).}  }
\details{
  Useful when testing for panmagic squares.  The first element is always
  the unbroken one (ie \code{[1,1]} to \code{[n,n]} if \code{nw.se} is
  \code{TRUE} and \code{[1,n]} to \code{[n,1]} if \code{nw.se} is
  \code{FALSE}.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{is.panmagic}}}
\examples{
diag.off(magic(10),nw.se=FALSE,offset=0)
diag.off(magic(10),nw.se=FALSE,offset=1)
}
\keyword{array}

\eof
\name{dimension}
\alias{dimension}
\title{Generalized row and col}
\description{
Given an array, returns an array of the same size whose elements
are sequentially numbered along the \eqn{i^{\rm th}}{i-th} dimension.

Used by \code{magiccube.2np1}.
}
\usage{
dimension(a, i)
}
\arguments{
  \item{a}{array to be converted}
  \item{i}{Number of the dimension}
}
\value{
An integer matrix with the same dimensions as \code{a}, with element 
\eqn{\left(n_1,n_2,\ldots n_d\right)^{\rm th}}{(n1,n2, ..., n_d)}
being \eqn{n_i}{n_i}.
}
\author{Robin K. S. Hankin}
\examples{
a <- array(0,c(3,3,2,2))
dimension(a,2)
(dimension(a,1)+dimension(a,2)+dimension(a,3)+dimension(a,4))\%\%2

}
\keyword{array}

\eof
 \name{eq}
\alias{eq}
\alias{ne}
\alias{gt}
\alias{lt} 
\alias{ge}
\alias{le}
\alias{\%eq\%}
\alias{\%ne\%}
\alias{\%gt\%}
\alias{\%lt\%}
\alias{\%ge\%}
\alias{\%le\%}
\title{Comparison of two magic squares}
\description{
Compares two magic squares according to Frenicle's method.  Mnemonic is
the old Fortran ``.GT."  (for ``Greater Than") comparison et seq.

To compare magic square \code{a} with magic square \code{b}, their
elements are compared in rowwise order: \code{a[1,1]} is compared with
\code{b[1,1]} , then \code{a[1,2]} with \code{b[1,2]}, up to
\code{a[n,n]}.  Consider the first element that is different, say
\code{[i,j]}.  Then \code{a<b} if \code{a[i,j]<b[i,j]}.

The generalization to hypercubes is straightforward: comparisons are
carried out natural order.

}
\usage{
eq(m1, m2)
ne(m1, m2)
gt(m1, m2)
lt(m1, m2)
ge(m1, m2)
le(m1, m2)
m1 \%eq\% m2
m1 \%ne\% m2
m1 \%gt\% m2
m1 \%lt\% m2
m1 \%ge\% m2
m1 \%le\% m2
}
\arguments{
  \item{m1}{First magic square}
  \item{m2}{Second magic square}
}
\author{Robin K. S. Hankin}
\note{Rather clumsy function definition due to the degenerate case of 
testing two identical matrices (\code{min(NULL)} is undefined).
}
\seealso{\code{\link{as.standard}}}
\examples{
magic(4) \%eq\% magic.4n(1)
eq(magic(4) , magic.4n(1))
}
\keyword{array}

\eof
\name{force.integer}
\alias{force.integer}
\title{Integerize array elements}
\description{
Returns an elementwise \code{as.integer}-ed array.  All magic squares
should have integer elements.
}
\usage{
force.integer(x)
}
\arguments{
  \item{x}{Array to be converted}
}
\author{Robin K. S. Hankin}
\keyword{array}

\eof
\name{hendricks}
\alias{hendricks}
\docType{data}
\title{A perfect magic cube due to Hendricks}
\description{
A perfect \eqn{8\times 8\times 8}{8*8*8} magic cube due to Hendricks}
\usage{data(hendricks)}
\source{\url{http://members.shaw.ca/hdhcubes/cube_perfect.htm#Definitions}}
\examples{
data(hendricks)
is.perfect(hendricks)
}
\keyword{datasets}
\keyword{array}

\eof
\name{is.magic}
\alias{is.magic}
\alias{is.panmagic}
\alias{is.semimagic}
\alias{is.associative}
\alias{is.normal}
\alias{is.mostperfect}
\alias{is.2x2.correct}
\alias{is.diagonally.correct}
  
\title{Various tests for the magicness of a square}
\description{
  Returns TRUE if the square is magic, semimagic, panmagic, associative,
  normal.  If argument \code{give.answers} is TRUE, also returns
  additional information about the sums.
}
\usage{
is.magic(m,give.answers=FALSE)
is.panmagic(m,give.answers=FALSE)
is.semimagic(m,give.answers=FALSE)
is.associative(m)
is.normal(m)
is.mostperfect(m,give.answers=FALSE)
is.2x2.correct(m,give.answers=FALSE)
is.diagonally.correct(m,give.answers=FALSE)
}
\arguments{
  \item{m}{The square to be tested}
  \item{give.answers}{Boolean, with TRUE meaning return additional
    information about the sums (see details).}
}
\details{
  A {\em semimagic square} is one all of whose row sums equal all its
  columnwise sums (ie the magic constant).

  A {\em magic square} is a semimagic square with the sum of both unbroken
  diagonals equal to the magic constant.

  A {\em panmagic square} is a magic square all of whose broken diagonals
  sum to the magic constant.  Ollerenshaw calls this a ``pandiagonal''
  square.

  A {\em most perfect} square has all 2-by-2 arrays anywhere within the square
  summing to \eqn{2S} where \eqn{S=n^2+1}{S=n^2+1}; and all pairs of
  integers \eqn{n/2}{n/2} distant along the same major (NW-SE) diagonal sum to
  \eqn{S}{S} (note that the \eqn{S}{S} used here differs
  from Ollerenshaw's because her squares are numbered starting at zero).
  The first condition is tested by  \code{\link{is.2x2.correct}} and the
  second by \code{\link{is.diagonally.correct}}.
  
  All most perfect squares are panmagic.

  A {\em normal square} is one that contains \eqn{n^2}{n^2} consecutive
  integers (typically starting at 0 or 1).  
}
\value{
  Returns TRUE if the square is semimagic, etc.
 
  If \code{give.answers} is taken as an argument and is TRUE, return a
  list of at least five elements.  The first element of the list is the
  answer: it is TRUE if the square is (semimagic, magic, panmagic).
  Elements 2-5 give the result of a call to \code{allsums()}, viz: rowwise
  and columwise sums; and broken major (ie NW-SE) and minor (ie NE-SW)
  diagonal sums.

  Function \code{is.diagonally.correct()} also returns the sums of
  elements distant \eqn{n/2}{n/2} along a major diagonal
  (\code{diag.sums}); and function \code{is.2x2.correct()} returns the
  sum of each \eqn{2\times 2}{2x2} submatrix (\code{tbt.sums}).
  Function \code{is.mostperfect()} returns both of these.


}
\references{\url{http://mathworld.wolfram.com/MagicSquare.html}}
\author{Robin K. S. Hankin}
\seealso{\code{\link{minmax}},\code{\link{is.perfect}},\code{\link{is.semimagichypercube}}}
\examples{
is.magic(magic(4))
f <- function(n){is.magic(magic(n))}
all(sapply(3:50,f))

is.panmagic(panmagic.4())
is.panmagic(panmagic.8())

data(Ollerenshaw)
is.mostperfect(Ollerenshaw)


proper.magic <- function(m){is.magic(m) & minmax(c(1,diff(sort(m))))}
proper.magic(magic(20))
}
\keyword{array}

\eof
\name{is.semimagichypercube}
\alias{is.semimagichypercube}
\alias{is.magichypercube}
\alias{is.perfect}
\title{magic hypercubes}
\description{
Returns \code{TRUE} if a hypercube is semimagic, magic, perfect
}
\usage{
is.semimagichypercube(a,give.answers=FALSE)
is.magichypercube(a,give.answers=FALSE)
is.perfect(a)
}
\arguments{
  \item{a}{The hypercube (array) to be tested}
  \item{give.answers}{Boolean, with TRUE meaning to also return
    the sums}
}
\details{
  (Although apparently non-standard, here a hypercube is defined to have
  dimension \eqn{d}{d} and order \eqn{n}{n}---and thus has \eqn{n^d}{n^d}
  elements).
  
  A {\em semimagic hypercube} has all ``rook's move'' sums equal to the magic
  constant (that is, each
  \eqn{\sum a[i_1,i_2,\ldots ,i_{r-1},,i_{r+1},\ldots,i_d]}{sum(a[i_1,i_2,\ldots,i_{r-1},,i_{r+1},\ldots,i_d])} with \eqn{1\leq r\leq d}{1 <= r <= d} is
  equal to the magic constant for all values of the \eqn{i}{i}'s).  In
  \code{is.semimagichypercube()}, if \code{give.answers}
  is \code{TRUE}, the sums returned are in the form of an array of
  dimension \code{c(rep(n,d-1),d)}.  The first \code{d-1} dimensions are
  the coordinates of the projection of the summed elements onto the surface
  hypercube.  The last dimension indicates the dimension along which the
  sum was taken over.

  A {\em magic hypercube} is a semimagic hypercube with all
  \eqn{2^{d-1}}{2^(d-1)} major (ie extreme point-to-extreme point)
  diagonals summing correctly.  Note that not all
  subhypercubes are necessarily magic! (for example, consider a
  5-dimensional magic hypercube \code{a}.  The square \code{b} defined
  by \code{a[1,1,1,,]} might not be magic: the diagonals of
  \code{b} are not covered by the definition of a magic hypercube).
  Some subhypercubes of a magic hypercube are not even semimagic: see
  below for an example.

  In \code{is.magichypercube()}, if argument \code{give.answers=TRUE}
  then a list is returned.  The first element of this list is Boolean
  with \code{TRUE} if the array is a magic hypercube.  The second
  element gives the rook's move sums as
  for \code{is.semimagichypercube()} and the third element gives an
  array of \eqn{2^d}{2^d} elements with index 1 meaning the sum is taken
  from elements in \code{a} with the \eqn{d^{\rm th}}{d-th} dimension
  running over \code{1:n} (index 2 means run over \code{n:1}).

  A {\em perfect magic hypercube} is a magic hypercube with all
  nonbroken diagonals summing correctly.  This is a bad-ass
  requirement for high dimensional hypercubes.

  Even in three dimensions, being perfect is pretty bad.  Consider a
  \eqn{5\times5\times 5}{5x5x5} (ie three dimensional), cube.  Say
  \code{a=magiccube.2np1(2)}.  Then the square defined by
  \code{sapply(1:n,function(i){a[,i,6-i]}, simplify=TRUE)}, which is a
  subhypercube of \code{a}, is not even semimagic: the
  rowsums are incorrect (the colsums must sum correctly because \code{a}
  is magic).  Note that the diagonals of this square are two of the
  ``extreme point-to-point'' diagonals of \code{a}.

  A {\em pandiagonal magic hypercube} (or sometimes just a {\em perfect
  hypercube}) has sums equal, and all sums of all diagonals,
  including broken diagonals, are equal.  This is one seriously bad-ass
  requirement.  I reckon that is a total of
  \eqn{\frac{1}{2}\left( 3^d-1\right)\cdot n^{d-1}}{(3^d-1)n^(d-1)/2}
  correct summations.  This is not coded up yet; I can't see how to do
  it in anything like a vectorized manner.

  The terminology in this area is pretty confusing.  
  }
  \references{
    \url{http://members.shaw.ca/hdhcubes/cube_perfect.htm#Cubes}
    "Generalized magic cubes", Dana Richards .  Mathematics
    Magazine, volume 53, number 2, March 1980.
  }
\author{Robin K. S. Hankin}
\seealso{\code{\link{is.magic}}, \code{\link{allsubhypercubes}},\code{\link{hendricks}}}
\examples{
library(abind)
is.semimagichypercube(magiccube.2np1(1))
is.semimagichypercube(magichypercube.4n(1,d=4))

is.perfect(magichypercube.4n(1,d=4))

#Now for a subhypercube of a magic hypercube that is not semimagic:
is.magic(allsubhypercubes(magiccube.2np1(1))[[10]])

data(hendricks)
is.perfect(hendricks)


#note that Hendricks's magic cube also has many broken diagonals summing
#correctly:

a <- allsubhypercubes(hendricks)
ld <- function(a){length(dim(a))}

jj <- unlist(lapply(a,ld))
f <- function(i){is.perfect(a[[which(jj==2)[i]]])}
all(sapply(1:sum(jj==2),f))

#but this is NOT enough to ensure that it is pandiagonal (but I
#think hendricks is pandiagonal).

}
\keyword{array}

\eof
\name{lozenge}
\alias{lozenge}
\title{Conway's lozenge algorithm for magic squares}
\description{
  Uses John Conway's lozenge algorithm to produce magic squares of
  singly-even order.
}
\usage{
lozenge(m)
}
\arguments{
  \item{m}{magic square of order \code{n=2m+1}}
}
\author{Robin Hankin}
\seealso{\code{\link{magic.4np2}}}
\examples{
lozenge(4)
all(sapply(1:10,function(n){is.magic(lozenge(n))}))
}
\keyword{array}

\eof
\name{magic.2np1}
\alias{magic.2np1}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Magic squares of odd order}
\description{
  Function to create magic squares of odd order 
}
\usage{
magic.2np1(m, ord.vec = c(-1, 1), break.vec = c(1, 0), start.point=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{creates a magic square of order \eqn{n=2m+1}{n=2m+1}}
  \item{ord.vec}{ordinary vector.  Default value of \code{c(-1,1) }
    corresponds to the usual northeast direction}
  \item{break.vec}{break vector.  Default of \code{c(1,0)} corresponds to the
    usual south direction}
  \item{start.point}{Starting position for the method (ie coordinates of
    unity).  Default value of NULL corresponds to row 1, column \code{m}.}
}
\references{Written up in loads of places.  The method (at least with
  the default ordinary and break vectors) seems to have been known since
  at least the Renaissance.

  Benson and Jacoby, and the Mathematica website, discuss the problem
  with nondefault ordinary and break vectors.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}, \code{\link{magic.prime}}}
\examples{
magic.2np1(1)
f <- function(n){is.magic(magic.2np1(n))}
all(sapply(1:20,f))

is.panmagic(magic.2np1(5,ord.vec=c(2,1),break.vec=c(1,3)))
}
\keyword{array}

\eof
\name{magic.4n}
\alias{magic.4n}
\title{Magic squares of order 4n}% \eqn{4n}{4n}}
\description{
Produces an associative magic square of order \eqn{4n}{4n} using the delta-x method.
}
\usage{
magic.4n(m, method="a")
}
\arguments{
  \item{m}{Order \eqn{n}{n} of the square is given by \eqn{n=4m}{n=4m}}
  \item{method}{string specifying which method to use.  Method ``a''
    appears to be uniformly better, and clearer.}
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}}
\examples{
magic.4n(4)
is.magic(magic.4n(5))
}
\keyword{array}


\eof
\name{magic.4np2}
\alias{magic.4np2}
\title{Magic squares of order 4n+2}
\description{
  Produces a magic square of order \eqn{4n+2}{4n+2} using
  Conway's ``LUX'' method
}
\usage{
magic.4np2(m)
}
\arguments{
  \item{m}{returns a magic square of order \eqn{n=4m+2}{n=4m+2}
    for \eqn{n>1}{n>1}, using Conway's
    ``LUX'' construction.}
}
\references{\url{http://mathworld.wolfram.com/MagicSquare.html}}
\author{Robin K. S. Hankin}
\note{I am not entirely happy with the method used: it's too complicated}
\seealso{\code{\link{magic}}}
\examples{
magic.4np2(1)
is.magic(magic.4np2(3))
  }
}
\keyword{array}

\eof
\name{magic.8}
\alias{magic.8}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Regular magic squares of order 8}
\description{
Returns all 90 regular magic squares of order 8
}
\usage{
magic.8(...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{\dots}{ignored  }
}
\value{
Returns an array of dimensions \code{c(8,8,90)} of which each slice is
an 8-by-8 magic square.
}
\references{\url{http://www.grogono.com/magic/index.php}}
\author{Robin K. S. Hankin}
\examples{
h <- magic.8()
h[,,1]

f <- function(i){is.magic(h[,,i])}
stopifnot(all(sapply(1:90,f)))
}
\keyword{array}

\eof
\name{magic}
\alias{magic}
\title{Creates magic squares}
\description{
  Creates normal magic squares of any order \eqn{>2}{>2}.  Uses
  the appropriate method depending on n modulo 4.
}
\usage{
magic(n)
}
\arguments{
  \item{n}{Order of magic square}
}
\details{
Calls either \code{magic.2np1()}, \code{magic.4n()},
or \code{magic.4np2()} depending on the value of \code{n}.  Returns a
magic square in standard format (compare the \code{magic.2np1()} et seq,
which return the square as generated by the direct algorithm).
}
\references{William H. Benson and Oswald Jacoby.  New recreations with
  magic squares, Dover 1976.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic.2np1}}, \code{\link{magic.prime}},
  \code{\link{magic.4np2}},
\code{\link{magic.4n}},\code{\link{lozenge}},
  \code{\link{as.standard}}, \code{\link{force.integer}}}
\examples{
magic(6)
all(sapply(3:10,function(n){is.magic(magic(n))}))
}
\keyword{array}

\eof
\name{magic.composite}
\alias{magic.composite}
\title{Composite magic squares}
\description{
  Gives a magic square that is a product of two magic squares.
}
\usage{
magic.composite(a, b)
}
\arguments{
  \item{a}{First magic square; if \code{a} is an integer, use \code{magic(a)}.}
  \item{b}{as above}
}
\details{
Produces a magic square of order order(a)*order(b).
}
\references{
  William H. Benson and Oswald Jacoby. New recreations with magic squares, Dover 1976.}
\author{Robin K. S. Hankin}
\examples{
magic.composite(3,4)
magic.composite(4,3)
}
\keyword{array}

\eof
\name{magic.constant}
\alias{magic.constant}
\title{Magic constant of a magic square}
\description{
  Returns the magic consant: that is, the common sum for all rows,
  columns and (broken) diagonals of a magic square
}
\usage{
magic.constant(n)
}
\arguments{
  \item{n}{Order of the square}
}
\details{
  Returns \eqn{n(n^2+1)/2}{n(n^2+1)/2}.  Note that this is correct for
  magic squares whose numbers run from \eqn{1\ldots n^2}{1...n^2}.  If the
  square is numbered \eqn{0\ldots n^2-1}{1 ... n^2-1} the correct magic
  constant would be \eqn{n(n^2-1)/2}{n(n^2-1)/2}, or
  \code{magic.constant(n)-n}. 
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}}
\examples{
magic.constant(4)
}
\keyword{array}

\eof
\name{magic.prime}
\alias{magic.prime}
\title{Magic squares prime order}
\description{
Produces magic squares of order using the standard method}
}
\usage{
magic.prime(n,i=2,j=3)
}
\arguments{
  \item{n}{The order of the square}
  \item{i}{row number of increment}
  \item{j}{column number of increment}
}
\details{
Claimed to work for prime order, but I've tried it (with the defaults
for \code{i} and \code{j}) for many  integers of the
form \eqn{6n+1}{6n+1} and \eqn{6n-1}{6n-1} and found no exceptions;
indeed, they all seem to be panmagic.  It is not clear to me 
when the process works and when it doesn't.  
}
\references{\url{http://www.magic-squares.de/general/general.html}}
\author{Robin K. S. Hankin}
\examples{
magic.prime(7)
f <- function(n){is.magic(magic.prime(n))}
all(sapply(6*1:30+1,f))
all(sapply(6*1:30-1,f))

is.magic(magic.prime(9,i=2,j=4),give.answers=TRUE)
magic.prime(7,i=2,j=4)
}
\keyword{array}

\eof
\name{magiccube.2np1}
\alias{magiccube.2np1}
\title{Magic cubes of order 2n+1}
\description{
Creates odd-order magic cubes
}
\usage{
magiccube.2np1(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{n=2m+1}
}
\references{website}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}}

\examples{

#try with m=3, n=2*3+1=7:

m <-7                     
n <- 2*m+1


apply(magiccube.2np1(m),c(1,2),sum)
apply(magiccube.2np1(m),c(1,3),sum)
apply(magiccube.2np1(m),c(2,3),sum)

#major diagonal checks out:
sum(magiccube.2np1(m)[matrix(1:n,n,3)])

#now other diagonals:
b <- c(-1,1)
f <- function(dir,v){if(dir>0){return(v)}else{return(rev(v))}}
g <- function(jj){sum(magiccube.2np1(m)[sapply(jj,f,v=1:n)])}
apply(expand.grid(b,b,b),1,g)  #each diagonal twice, once per direction.
}

\keyword{array}


\eof
\name{magiccubes}
\alias{magiccubes}
\docType{data}
\title{Magic cubes of order 3}
\description{
A list of four elements listing each fundamentally different magic cube
of order 3
}
\usage{data(magiccubes)}
\source{
Originally discovered by Hendricks
}
\references{
  \url{http://members.shaw.ca/hdhcubes/cube_perfect.htm}
}
\examples{
data(magiccubes)
magiccubes$a1
sapply(magiccubes,is.magichypercube)
}
\keyword{datasets}

\eof
\name{magichypercube.4n}
\alias{magichypercube.4n}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Magic hypercubes of order 4n}
\description{
Returns magic hypercubes of order 4n and any dimension.
}
\usage{
magichypercube.4n(m, d = 3)
}
%- maybe also 'usage' for other objects documented here. 
\arguments{
  \item{m}{Magic hypercube produced of order \eqn{n=4m}{n=4m}}
  \item{d}{Dimensionality of cube}
}
\details{
Uses  a rather kludgy (but vectorized) method.  I am not 100\% sure that
the method does in fact produce magic squares for all orders and dimensions.
}
\author{Robin K. S. Hankin}
\examples{
magichypercube.4n(1,d=4)
magichypercube.4n(2,d=3)
}
\keyword{array}

\eof
\name{magicplot}
\alias{magicplot}
\title{Joins consecutive numbers of a magic square.}
\description{
A nice way to graphically represent normal magic squares.  Lines are
plotted to join successive numbers from 1 to \eqn{n^2}{n^2}.
Many magic squares have attractive such plots.
}
\usage{
magicplot(m, number = TRUE, do.circuit = FALSE, ...)
}
\arguments{
  \item{m}{Magic square to be plotted.}
  \item{number}{Boolean variable with TRUE meaning to include the
    numbers on the plot}
  \item{do.circuit}{Boolean variable with TRUE meaning to include the
    line joining \eqn{n^2}{n^2} to 1}
  \item{\dots}{Extra parameters passed to \code{plot()}}
}
\author{Robin K. S. Hankin}
\examples{
magicplot(magic.4n(2))
}
\keyword{array}


\eof
\name{minmax}
\alias{minmax}
\title{are all elements of a vector identical?}
\description{
Returns TRUE if and only if all elements of a vector are identical.
}
\usage{
minmax(x)
}
\arguments{
  \item{x}{Vector to be tested}
}
\author{Robin K. S. Hankin}
\seealso{is.magic()}
\keyword{array}


\eof
\name{notmagic.2n}
\alias{notmagic.2n}
\title{An unmagic square}
\description{
  Returns a square of order \eqn{n=2m}{n=2m} that has been claimed to
  be magic, but isn't.
}
\usage{
notmagic.2n(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{Order of square is \eqn{n=2m}{n=2m}}
}
\references{``Magic Squares and Cubes'', Andrews, (book)}
\author{Robin K. S. Hankin}
\note{This took me a whole evening to code up.  And I was quite pleased
  with the final vectorized form: it matches Andrews's (8 by 8) example
  square exactly.  What a crock.}
\examples{
  notmagic.2n(4)
  is.magic(notmagic.2n(4))
  is.semimagic(notmagic.2n(4))

}
\keyword{array}


\eof
\name{panmagic.4}
\alias{panmagic.4}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Panmagic squares of order 4}
\description{
Creates all fundamentally different panmagic squares of order 4.
}
\usage{
panmagic.4(vals = 2^(0:3))
}
\arguments{
  \item{vals}{a length four vector giving the values which are combined
    in each of the \eqn{2^4}{2^4} possible ways.  Thus
    \code{vals=2^sample(0:3)} always gives a normal square
    (0-15 in binary).}
}

\references{\url{http://www.grogono.com/magic/index.php}}
\author{Robin K. S. Hankin}
\examples{
panmagic.4()
panmagic.4(2^c(1,3,2,0))
panmagic.4(10^(0:3))
}
\keyword{array}

\eof
\name{panmagic.8}
\alias{panmagic.8}
\title{Panmagic squares of order 8}
\description{
Produces each of a wide class of order 8 panmagic squares
}
\usage{
panmagic.8(chosen = 1:6, vals = 2^(0:5))
}
\arguments{
  \item{chosen}{Which of the magic carpets are used in combination}
  \item{vals}{The values combined to produce the magic square.  Choosing
  \code{0:5} gives a normal magic square.}
}
\references{\url{http://www.grogono.com/magic/index.php}}
\author{Robin K. S. Hankin}
\note{
  Not all choices for \code{chosen} give normal magic squares.  There
  seems to be no clear pattern.  See website in references for details.
}
\seealso{\code{\link{panmagic.4}}}
\examples{
is.panmagic(panmagic.8(chosen=2:7))
is.normal(panmagic.8(chosen=2:7))
is.normal(panmagic.8(chosen=c(1,2,3,6,7,8)))

#to see the twelve basis magic carpets, use "chosen" an integer
#between 1 and 12, with vals=1:

panmagic.8(chosen=1,vals=1)-1
image(panmagic.8(chosen=12,vals=1))

}
\keyword{array}


\eof
\name{perfectcube5}
\alias{perfectcube5}
\docType{data}
\title{A perfect magic cube of order 5}
\description{
A perfect cube of order 5, due to Trump and Boyer
}
\usage{data(perfectcube5)}
\examples{
data(perfectcube5)
is.perfect(perfectcube5)
}
\keyword{datasets}

\eof
\name{perfectcube6}
\alias{perfectcube6}
\docType{data}
\title{A perfect cube of order 6}
\description{
A perfect cube of order 6 originally due to Trump
}
\usage{data(perfectcube6)}
\examples{
data(perfectcube6)
is.perfect(perfectcube6)
is.magichypercube(perfectcube6[2:5,2:5,2:5])
}
\keyword{datasets}

\eof
\name{process}
\alias{process}
\title{Force index arrays into range}
\description{
Forces an (integer) array to have entries in the range 1-n, by (i)
taking the entries modulo n, then (ii) setting zero elements to n.
Useful for modifying index arrays into a form suitable for use with
magic squares.}
\usage{
process(x, n)
}
\arguments{
  \item{x}{Index array to be processed}
  \item{n}{Modulo of arithmetic to be used}
}
\author{Robin K. S. Hankin}
\examples{
# extract the broken diagonal of magic.2np1(4) that passes
# through element [1,5]:

a <- magic.2np1(4)
index <- t(c(1,5)+rbind(1:9,1:9))
a[process(index,9)]
}
\keyword{array}

\eof
